#ifndef _declaration_h
#define _declaration_h

unsigned int nbStates = 5;

unsigned nbRules = 216;  // = 6^3

unsigned int nbFree = 85; // = 4^2 + 4^3 + 4^2 - 3 - 3 - 2 - 2 - 1 
unsigned int * authorised = new unsigned[nbFree];

// notation pour certains états
int BORD   = nbStates;
int FIRE   = nbStates - 1;
int GEN    = 1;
int NONGEN = 2;
int REPOS  = 0;


void authorisedRules() {
  unsigned int k = 0;

  // a gauche
  for(int c = 0; c < FIRE; c++)
    for(int d = 0; d < FIRE; d++) {
      if ( !((c == REPOS && d == REPOS) ||
	     (c == GEN   && d == GEN)   ||
	     (c == GEN   && d == REPOS)) ) { 
	authorised[k] = BORD * 36 + c * 6 + d;
	k++;
      }
    }

  // au centre
  for(int g = 0; g < FIRE; g++)
    for(int c = 0; c < FIRE; c++)
      for(int d = 0; d < FIRE; d++) {
	if ( !((g == REPOS  && c == REPOS && d == REPOS) ||
	       (g == GEN    && c == GEN   && d == GEN)   ||
	       (g == GEN    && c == REPOS && d == REPOS) ||
	       (g == NONGEN && c == REPOS && d == REPOS)) ) { 
	  authorised[k] = g * 36 + c * 6 + d;
	  k++;
	}
      }

  // a droite
  for(int g = 0; g < FIRE; g++)
    for(int c = 0; c < FIRE; c++) {
      if ( !((g == REPOS  && c == REPOS) ||
	     (g == GEN    && c == GEN  ) ||
	     (g == GEN    && c == REPOS) ||
	     (g == NONGEN && c == REPOS)) ) { 
	authorised[k] = g * 36 + c * 6 + BORD;
	k++;
      }
    }

  //  std::cout << k << std::endl;
}

void setRule(Regle rules, int g, int c, int d, int r) {
  rules[g * 36 + c * 6 + d] = r;
}

pair<unsigned int, unsigned int> ** neighborhood;
unsigned int nbNeighbor;

void neighborhoodInit() {
  nbNeighbor = nbFree * 3;

  neighborhood = new pair<unsigned int, unsigned int>*[nbNeighbor];
  for(unsigned i = 0; i < nbFree; i++) {
    neighborhood[ 3*i     ] = new pair<unsigned int, unsigned int>(authorised[i], 1);
    neighborhood[ 3*i + 1 ] = new pair<unsigned int, unsigned int>(authorised[i], 2);
    neighborhood[ 3*i + 2 ] = new pair<unsigned int, unsigned int>(authorised[i], 3);
  }
}

void permutation(unsigned int i, unsigned int j) {
  pair<unsigned int, unsigned int> * tmp = neighborhood[i];
  neighborhood[i] = neighborhood[j];
  neighborhood[j] = tmp;
}

void save(Regle rules, char * fileName) {
  fstream file(fileName, fstream::out | fstream::app);

  if (file) {
    file << rules[0] ;
    for(int i = 1; i < nbRules; i++)
      file << " " << rules[i] ;
    file << endl;
    file.close();
  } else {
    cerr << "save: Impossible d'ouvrir " << fileName << endl;
  }
}

#endif
